import {FC, useCallback, useEffect, useState} from "react";
import {Image, Picker, View} from "@tarojs/components";
import styles from './offline.module.scss'
import Taro from "@tarojs/taro";
import {curriculum} from "@/api";
import {formatDate} from "@/utils/time";
import {getSetting, authorize} from "@tarojs/taro";
import PopPut from "@/components/popPut/popPut";
import MyButton from "@/components/button/MyButton";

const Offline: FC = () => {
  const [manages, setManages] = useState<Manage[]>([])
  const [start, setStart] = useState<string>(formatDate(new Date(), "YY-MM-dd 08:00:00"))
  const [end, setEnd] = useState<string>(formatDate(new Date(), "YY-MM-dd 18:00:00"))
  const [tempFilePath, setTempFilePath] = useState<string | null>(null);
  const [error, setError] = useState<string | null>(null)
  const [depid, setDepid] = useState<number | null>(null)
  const [isDownloading, setDownloading] = useState(false)

  Taro.useLoad(() => {
    curriculum.department().then(res => {
      setManages(res.data)
    })
  })

  useEffect(() => {
    if (!depid) {
      return
    }

    if (isDownloading) {
      return;
    }

    setDownloading(true)

    const startTime = new Date(start).getTime()
    const endTime = new Date(end).getTime()
    const path = encodeURIComponent("/pages/meeting/meeting")

    const qrcodeUrl = `${process.env.TARO_APP_API}/official/qrcode?depid=${depid}&start_time=${startTime}&end_time=${endTime}&path=${path}&t=${Date.now()}`

    if (process.env.TARO_ENV === 'h5') {
      setTempFilePath(qrcodeUrl)
    } else {
      Taro.downloadFile({
        url: qrcodeUrl,
        success(res) {
          setTempFilePath(res.tempFilePath)
        },
        fail() {
          setError("请求失败");
        },
        complete() {
          setDownloading(false)
        }
      })
    }
  }, [depid])

  function change(e) {
    setDepid(() => manages[Number(e.detail.value)]?.id)
  }

  const handleWriteFile = useCallback(() => {
    if (tempFilePath == null) {
      Taro.showToast({title: '下载失败', icon: 'error'})
    } else if (process.env.TARO_ENV !== 'h5') {
      Taro.saveImageToPhotosAlbum({
        filePath: tempFilePath,
        success() {
          Taro.showModal({title: '下载成功'})
        },
        fail() {
          Taro.showToast({title: '下载失败', icon: 'error'})
        }
      })
    } else {
      Taro.showToast({title: '请截屏', icon: 'error'})
    }
  }, [tempFilePath])

  const handleSaveCode = useCallback(() => {
    getSetting({
      success: function ({authSetting}) {
        //没有权限则申请
        if (!authSetting["scope.writePhotosAlbum"]) {
          authorize({
            scope: "scope.writePhotosAlbum",
            success: () => {
              //存储二维码
              handleWriteFile()
            },
          });
        } else {
          handleWriteFile()
        }
      },
    });
  }, [tempFilePath]);


  return (
    <View className={styles.page}>

      <Picker mode="date" value={start} onChange={(e) => setStart(e.detail.value + ' 8:00:00')} name='start'>
        <PopPut title='开始时间' content={start}/>
      </Picker>


      <Picker mode="date" value={end} onChange={(e) => setEnd(e.detail.value + ' 18:00:00')} name='end'>
        <PopPut title='结束时间' content={end}/>
      </Picker>

      <View>
        <Picker mode='selector' range={manages} onChange={change} rangeKey='name'>
          <PopPut title='部门' content={manages?.find(x => x.id === depid)?.name}/>
        </Picker>
      </View>

      {tempFilePath && <View className='text-center'>
        <Image
          src={tempFilePath}
          mode='widthFix'
          onLoad={() => setError(null)}
          onError={(e) => setError(e.detail.errMsg)}
          style={{width: '80%'}}
        />
        {error && <View>{error}</View>}`
        <MyButton onClick={handleSaveCode}>下载</MyButton>
      </View>}
    </View>
  )
}

export default Offline
